/*
    Document   : fobi.djangocms_admin_style_theme.css
    Created on : July 25, 2014, 01:55:27 AM
    Author     : Artur Barseghyan (artur.barseghyan@gmail.com)
    Description:
        `django-fobi` "djangocms_admin_style_theme" theme main styles.
*/
/*
 * Style tweaks
 * --------------------------------------------------
 */
html,
body {
  overflow-x: hidden; /* Prevent scroll on narrow devices */
}
body.fobi-home {
    padding-top: 0;
}

.form-element-position {
    /*position: absolute;
    left: -9999px;*/
}

/*
 * Off Canvas
 * --------------------------------------------------
 */
@media screen and (max-width: 767px) {
  .row-offcanvas {
    position: relative;
    -webkit-transition: all 0.25s ease-out;
    -moz-transition: all 0.25s ease-out;
    transition: all 0.25s ease-out;
  }

  .row-offcanvas-right
  .sidebar-offcanvas {
    right: -50%; /* 6 columns */
  }

  .row-offcanvas-left
  .sidebar-offcanvas {
    left: -50%; /* 6 columns */
  }

  .row-offcanvas-right.active {
    right: 50%; /* 6 columns */
  }

  .row-offcanvas-left.active {
    left: 50%; /* 6 columns */
  }

  .sidebar-offcanvas {
    position: absolute;
    top: 0;
    width: 50%; /* 6 columns */
  }
}

/*
 * Plugin specific
 * --------------------------------------------------
 */
