/*
    Document   : fobi.plugin.slider-bootstrap3-widget.js
    Created on : Oct 19, 2016, 03:30:35 AM
    Author     : Artur Barseghyan (artur.barseghyan@gmail.com)
    Description:
        `bootstrap3` theme slider plugin scripts.
*/
;

$(document).ready(function() {
    $('.slider').bootstrapSlider();
    $('.slider-no-background').bootstrapSlider();
});
