    <h3>{% block form_page_title %}{% endblock %}</h3>
    <div class="section-container tabs" data-section="">
      <section class="section">
        <div class="content" data-slug="panel1">
          <form method="{% block form_method %}post{% endblock %}" action="{% block form_action %}{% if form_entry.action %}{{ form_entry.action }}{% else %}{{ request.path }}{% endif %}{% endblock %}" {% block form_enctype %}enctype="multipart/form-data"{% endblock %} class="{% block form_html_class %}{% endblock %}" {% block form_extra_attrs %}{% endblock %}>
            {% csrf_token %}
            {% include fobi_theme.form_snippet_template_name %}
            <div class="{% block form_button_outer_wrapper_html_class %}{% endblock %}">
              <div class="{% block form_button_wrapper_html_class %}{% endblock %}">
                {% block form_buttons %}
                <button type="submit" class="{% block form_primary_button_html_class %}{% endblock %}">{% block form_primary_button_text %}{% endblock %}</button>
                {% endblock form_buttons %}
              </div>
            </div>
          </form>
        </div>
      </section>
    </div>
